#!/usr/bin/env python

# We'll use json.loads for parsing the JSON data.
import json

# Import the citeproc-py classes we'll use below.
from citeproc import CitationStylesStyle, CitationStylesBibliography
from citeproc import Citation, CitationItem
from citeproc import formatter
from citeproc.source.json import CiteProcJSON


# The following JSON data describes 5 references picked from the CSL test suite.

json_input = """
[
    {
        "id": "ITEM-1",
        "issued": {
            "date-parts": [[1987,  8,  3],
                           [2003, 10, 23]]
        },
        "title": "Ignore me",
        "type": "book"
    },
    {
      "id" : "ITEM-2",
      "page" : "1-7",
      "type" : "article-journal",
      "issued" : {
        "date-parts": [[2006]]
      }
    },
    {
        "author": [
            {
                "family": "Doe",
                "given": "John"
            }
        ],
        "id": "ITEM-3",
        "issued": {
            "date-parts": [["1965", "6", "1"]]
        },
        "title": "His Anonymous Life",
        "type": "book"
    },
    {
        "author": [
            {
                "family": "Grignon",
                "given": "Cyril"
            },
                        {
                "family": "Sentenac",
                "given": "Corey"
            }
        ],
        "id": "ITEM-4",
        "issued": {
            "date-parts": [[2000]]
       },
        "type": "book"
    },
    {
        "id": "ITEM-5",
        "title":"Boundaries of Dissent: Protest and State Power in the Media Age",
        "author": [
                {
                        "family": "D'Arcus",
                        "given": "Bruce",
                        "static-ordering": false
                }
        ],
        "publisher": "Routledge",
        "publisher-place": "New York",
        "issued": {
            "date-parts":[[2006]]
        },
        "type": "book",
        "URL": "http://www.test01.com"
    }
]
"""


# Parse the JSON input using json.loads()
# (parsing from a file object can be done with json.load)

json_data = json.loads(json_input)

# Process the JSON data to generate a citeproc-py BibliographySource.

bib_source = CiteProcJSON(json_data)
##for key, entry in bib_source.items():
##    print(key)
##    for name, value in entry.items():
##        print('   {}: {}'.format(name, value))

# load a CSL style (from the current directory)

bib_style = CitationStylesStyle('harvard1', validate=False)

# Create the citeproc-py bibliography, passing it the:
# * CitationStylesStyle,
# * BibliographySource (CiteProcJSON in this case), and
# * a formatter (plain, html, or you can write a custom formatter)

bibliography = CitationStylesBibliography(bib_style, bib_source, formatter.html)


# Processing citations in a document needs to be done in two passes as for some
# CSL styles, a citation can depend on the order of citations in the
# bibliography and thus on citations following the current one.
# For this reason, we first need to register all citations with the
# CitationStylesBibliography.

citation1 = Citation([CitationItem('ITEM-3')])
citation2 = Citation([CitationItem('ITEM-1'), CitationItem('ITEM-2')])
citation3 = Citation([CitationItem('ITEM-4')])
citation4 = Citation([CitationItem('ITEM-5')])
citation5 = Citation([CitationItem('MISSING')])

bibliography.register(citation1)
bibliography.register(citation2)
bibliography.register(citation3)
bibliography.register(citation4)
bibliography.register(citation5)


# In the second pass, CitationStylesBibliography can generate citations.
# CitationStylesBibliography.cite() requires a callback function to be passed
# along to be called in case a CitationItem's key is not present in the
# bibliography.

def warn(citation_item):
    print("WARNING: Reference with key '{}' not found in the bibliography."
          .format(citation_item.key))

print('Citations')
print('---------')

print(bibliography.cite(citation1, warn))
print(bibliography.cite(citation2, warn))
print(bibliography.cite(citation3, warn))
print(bibliography.cite(citation4, warn))
print(bibliography.cite(citation5, warn))


# And finally, the bibliography can be rendered.

print('')
print('Bibliography')
print('------------')

for item in bibliography.bibliography():
    print(str(item))



#!/usr/bin/env python

# The references are parsed from a BibTeX database, so we import the
# corresponding parser.
from citeproc.source.bibtex import BibTeX

# Import the citeproc-py classes we'll use below.
from citeproc import CitationStylesStyle, CitationStylesBibliography
from citeproc import formatter
from citeproc import Citation, CitationItem


# Parse the BibTeX database.

bib_source = BibTeX('xampl.bib')


# load a CSL style (from the current directory)

bib_style = CitationStylesStyle('harvard1', validate=False)


# Create the citeproc-py bibliography, passing it the:
# * CitationStylesStyle,
# * BibliographySource (BibTeX in this case), and
# * a formatter (plain, html, or you can write a custom formatter)

bibliography = CitationStylesBibliography(bib_style, bib_source,
                                          formatter.plain)


# Processing citations in a document needs to be done in two passes as for some
# CSL styles, a citation can depend on the order of citations in the
# bibliography and thus on citations following the current one.
# For this reason, we first need to register all citations with the
# CitationStylesBibliography.

citation1 = Citation([CitationItem('whole-collection')])
citation2 = Citation([CitationItem('whole-set'), CitationItem('misc-full')])
citation3 = Citation([CitationItem('techreport-full')])
citation4 = Citation([CitationItem('mastersthesis-minimal')])
citation5 = Citation([CitationItem('inproceedings-full'),
                      CitationItem('unpublished-full')])

bibliography.register(citation1)
bibliography.register(citation2)
bibliography.register(citation3)
bibliography.register(citation4)
bibliography.register(citation5)


# In the second pass, CitationStylesBibliography can generate citations.
# CitationStylesBibliography.cite() requires a callback function to be passed
# along to be called in case a CitationItem's key is not present in the
# bibliography.

def warn(citation_item):
    print("WARNING: Reference with key '{}' not found in the bibliography."
          .format(citation_item.key))


print('Citations')
print('---------')

print(bibliography.cite(citation1, warn))
print(bibliography.cite(citation2, warn))
print(bibliography.cite(citation3, warn))
print(bibliography.cite(citation4, warn))
print(bibliography.cite(citation5, warn))


# And finally, the bibliography can be rendered.

print('')
print('Bibliography')
print('------------')

for item in bibliography.bibliography():
    print(str(item))




import os

from warnings import warn

from lxml import etree

from . import SCHEMA_PATH, LOCALES_PATH, STYLES_PATH
from .model import CitationStylesElement
from .formatter import html


class CitationStylesXML(object):
    def __init__(self, f, validate=True):
        lookup = etree.ElementNamespaceClassLookup()
        namespace = lookup.get_namespace('http://purl.org/net/xbiblio/csl')
        namespace[None] = CitationStylesElement
        namespace.update(dict([(cls.__name__.replace('_', '-').lower(), cls)
                               for cls in CitationStylesElement.__subclasses__()]))

        self.parser = etree.XMLParser(remove_comments=True, encoding='UTF-8',
                                      no_network=True)
        self.parser.set_element_class_lookup(lookup)
        self.xml = etree.parse(f, self.parser)#, base_url=".")
        if validate:
            self.schema = etree.RelaxNG(etree.parse(SCHEMA_PATH))
            if not self.schema.validate(self.xml):
                err = self.schema.error_log
                #raise Exception("XML file didn't pass schema validation:\n%s" % err)
                warn("XML file didn't pass schema validation:\n%s" % err)
                # TODO: proper error reporting
        self.root = self.xml.getroot()


class CitationStylesLocale(CitationStylesXML):
    def __init__(self, locale, validate=True):
        locale_path = os.path.join(LOCALES_PATH, 'locales-{}.xml'.format(locale))
        try:
            super(CitationStylesLocale, self).__init__(locale_path,
                                                       validate=validate)
        except IOError:
            raise ValueError("'{}' is not a known locale".format(locale))


class CitationStylesStyle(CitationStylesXML):
    def __init__(self, style, locale=None, validate=True):
        style_src = None

        try:
            # lxml can take file-like objects, so check if we are dealing with
            # strings first
            if not isinstance(style, str):
                pass  # pass through for a generic handling below
            # It could still be a URL etc, first check if style is a path that exists
            elif os.path.exists(style):
                style_src = style
            else:
                # Try bundled styles
                bundled_path = os.path.join(STYLES_PATH, f'{style}.csl')
                if os.path.exists(bundled_path):
                    style_src = bundled_path
                else:
                    # Try to load from citeproc-py-styles if available
                    try:
                        import citeproc_styles
                    except ImportError:
                        # citeproc-py-styles not installed, raise with helpful message
                        raise ValueError(
                            f"'{style}' not found in bundled styles ({STYLES_PATH}). "
                            f"To access more styles, install the citeproc-py-styles package with: "
                            f"pip install citeproc-py-styles"
                        )
                    try:
                        from citeproc_styles import get_style_filepath
                        style_src = get_style_filepath(style)
                    except (KeyError, FileNotFoundError):
                        # Style not found in citeproc-py-styles
                        raise ValueError(
                            f"'{style}' not found in bundled styles ({bundled_path}) "
                            f"or in citeproc-py-styles package"
                        )
        except TypeError:
            pass

        if style_src is None:
            if style:
                # Assume it's a file-like object or string containing XML data
                style_src = style
            else:
                # If we couldn't find the style anywhere, raise an error
                raise ValueError(f"'{style}' is not a known style")

        try:
            super(CitationStylesStyle, self).__init__(
                style_src, validate=validate)
        except IOError:
            raise ValueError(f"'{style}' is not a known style")
        if locale is None:
            locale = self.root.get('default-locale', 'en-US')
        self.root.set_locale_list(locale, validate=validate)

    def has_bibliography(self):
        return self.root.bibliography is not None

    def render_citation(self, citation, cites, callback=None):
        return self.root.citation.render(citation, cites, callback)

    def sort_bibliography(self, citation_items):
        return self.root.bibliography.sort(citation_items)

    def render_bibliography(self, citation_items):
        return self.root.bibliography.render(citation_items)


class CitationStylesBibliography(object):
    def __init__(self, style, source, formatter=html):
        self.style = style
        self.source = source
        self.formatter = self.style.root.formatter = formatter
        self.keys = []
        self.items = []
        self._cites = []

    def register(self, citation, callback=None):
        citation.bibliography = self
        for item in citation.cites:
            if item.key in self.source:
                if item.key not in self.keys:
                    self.keys.append(item.key)
                    self.items.append(item)
            elif callback is not None:
                callback(item)

    def sort(self):
        self.items = self.style.sort_bibliography(self.items)
        self.keys = [item.key for item in self.items]

    def cite(self, citation, callback):
        return self.style.render_citation(citation, self._cites, callback)

    def bibliography(self):
        return self.style.render_bibliography(self.items)



# coding: utf-8
from citeproc import (
    Citation,
    CitationItem,
    CitationStylesBibliography,
    CitationStylesStyle,
    formatter,
    source,
)
from unittest import TestCase
from citeproc.source.bibtex.bibparse import BibTeXParser

template = {
        "type": "book",
        "title": "La Rettorica",
        "author": [{"literal": "Brunetto Latini"}],
        "issued": {"date-parts": [[1968]]},
        "editor": [{"literal": "Francesco Maggini"}],
        "publisher": "Le Monnier",
        "location": "Firenze",
    }


def _pp(string):
    return string.split(" ")[5]


class TestBibliographyGeneration(TestCase):
    def test_generate(self):
        expected_ordinals = {
            "en-US": {0: "1st", 1: "2nd", 3: "4th", 19: "20th"},
            "de-DE": {0: "1.", 9: "10."},
            "es-ES": {0: "1.ª"},
            "fr-FR": {0: "1ʳᵉ", 9: "10ᵉ"},
            "hi-IN": {0: "1वाँ"},
            "it-IT": {0: "1º"}
        }

        for lg, ordinals in expected_ordinals.items():
            bib_style = CitationStylesStyle("harvard1", lg)
            entries = []
            for edition in range(1, 26):
                template["id"] = str(edition)
                template["edition"] = edition
                entries.append(template.copy())
            bib = source.json.CiteProcJSON(entries)
            bibliography = CitationStylesBibliography(bib_style, bib, formatter.plain)
            citations = [CitationItem(str(x)) for x in range(1, 26)]
            bibliography.register(Citation(citations))
            generated_ordinals = [_pp(x) for x in bibliography.style.render_bibliography(citations)]

            for index, expected_value in ordinals.items():
                assert generated_ordinals[index] == expected_value, f"Failed for {lg} at index {index}. Expected: {expected_value}, Got: {generated_ordinals[index]}"

    def test_ordinals_with_embedded_locale(self):
        """chicago-author-date has an embedded <locale xml:lang="en"> that defines
        editor/translator terms but no ordinal terms.  Ordinals must still be
        resolved from the file-based en-US locale data.
        Regression test for https://github.com/citeproc-py/citeproc-py/issues/183."""
        bib_style = CitationStylesStyle("chicago-author-date", "en-US")

        entries = [
            {
                "id": str(n),
                "type": "book",
                "title": "Some Book",
                "author": [{"family": "Smith", "given": "John"}],
                "issued": {"date-parts": [[2020]]},
                "publisher": "Publisher",
                "publisher-place": "New York",
                "edition": n,
            }
            for n in range(1, 5)
        ]

        bib = source.json.CiteProcJSON(entries)
        bibliography = CitationStylesBibliography(bib_style, bib, formatter.plain)
        citations = [CitationItem(str(x)) for x in range(1, 5)]
        bibliography.register(Citation(citations))
        rendered = bibliography.style.render_bibliography(citations)

        # The ordinal edition token is the 6th whitespace-delimited word, e.g.:
        # "Smith, John. 2020. Some Book. 2nd ed. New York: Publisher."
        ordinals = [r.split()[5] for r in rendered]
        assert ordinals[0] == "1st", f"Expected '1st', got '{ordinals[0]}'"
        assert ordinals[1] == "2nd", f"Expected '2nd', got '{ordinals[1]}'"
        assert ordinals[2] == "3rd", f"Expected '3rd', got '{ordinals[2]}'"
        assert ordinals[3] == "4th", f"Expected '4th', got '{ordinals[3]}'"



#!/usr/bin/env python

"""
Setup script for citeproc-py
"""

import io
import os
import re
import sys

from datetime import datetime
from subprocess import Popen, PIPE
from setuptools import setup, find_packages
from setuptools.command.build_py import build_py
from setuptools.command.develop import develop
import versioneer

PACKAGE = 'citeproc'
BASE_PATH = os.path.dirname(os.path.abspath(__file__))
PACKAGE_ABSPATH = os.path.join(BASE_PATH, PACKAGE)
VERSION_FILE = os.path.join(PACKAGE_ABSPATH, 'version.py')


# All external commands are relative to BASE_PATH
os.chdir(BASE_PATH)


def long_description():
    with open(os.path.join(BASE_PATH, 'README.md'), encoding='utf-8') as readme:
        result = readme.read()
    result += '\n\n'
    with open(os.path.join(BASE_PATH, 'CHANGELOG.md'), encoding='utf-8') as changes:
        result += changes.read()
    return result


CSL_SCHEMA_RNC = 'citeproc/data/schema/csl.rnc'

def convert_rnc():
    import rnc2rng

    filename_root, _ = os.path.splitext(CSL_SCHEMA_RNC)
    root = rnc2rng.load(CSL_SCHEMA_RNC)
    with io.open(filename_root + '.rng', 'w', encoding='utf-8') as rng:
        rnc2rng.dump(root, rng)


class custom_build_py(build_py):
    def run(self):
        convert_rnc()
        build_py.run(self)


class custom_develop(develop):
    def run(self):
        convert_rnc()
        develop.run(self)


setup(
    name='citeproc-py',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass({'build_py': custom_build_py, 'develop': custom_develop}),
    packages=find_packages(),
    package_data={PACKAGE: ['data/locales/*.xml',
                            'data/locales/locales.json',
                            'data/schema/*.rng',
                            'data/styles/*.csl']},
    python_requires='>=3.9',
    scripts=['bin/csl_unsorted'],
    setup_requires=['rnc2rng>=2.6.1,!=2.6.2'],
    install_requires=['lxml'],
    extras_require={
        'full': ['citeproc-py-styles'],
        'tests': ['citeproc-py-styles'],
    },
    provides=[PACKAGE],
    #test_suite='nose.collector',

    author='Brecht Machiels',
    author_email='brecht@mos6581.org',
    description='Citations and bibliography formatter',
    long_description=long_description(),
    long_description_content_type='text/markdown',
    url='https://github.com/brechtm/citeproc-py',
    keywords='csl citation html rst bibtex xml',
    license='BSD-2-Clause-Views',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Environment :: Other Environment',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Legal Industry',
        'Intended Audience :: Other Audience',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: 3.13',
        'Topic :: Documentation',
        'Topic :: Printing',
        'Topic :: Software Development :: Documentation',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



"""
Tests for flexible style loading from citeproc-py-styles package
"""

import os
import sys
import tempfile
import unittest
from unittest.mock import patch, MagicMock

from citeproc import CitationStylesStyle, STYLES_PATH

class TestStyleLoading(unittest.TestCase):
    """Test suite for flexible style loading"""

    def test_bundled_style_loads(self):
        """Test that bundled harvard1 style loads correctly"""
        style = CitationStylesStyle('harvard1', validate=False)
        self.assertIsNotNone(style)

    def test_full_path_style_loads(self):
        """Test that a style specified with full path loads correctly"""
        harvard_path = os.path.join(STYLES_PATH, 'harvard1.csl')
        style = CitationStylesStyle(harvard_path, validate=False)
        self.assertIsNotNone(style)

    def test_nonexistent_style_without_citeproc_py_styles(self):
        """Test error message when style not found and citeproc-py-styles not installed"""
        with patch('citeproc.frontend.os.path.exists') as mock_exists:
            # Make sure the style doesn't exist as a file or bundled
            mock_exists.side_effect = lambda path: False

            # Mock the import to fail
            with patch.dict('sys.modules', {'citeproc_styles': None}):
                with self.assertRaises(ValueError) as context:
                    CitationStylesStyle('nonexistent-style', validate=False)

                error_msg = str(context.exception)
                self.assertIn('nonexistent-style', error_msg)
                self.assertIn('not found in bundled styles', error_msg)
                self.assertIn('pip install citeproc-py-styles', error_msg)

    def test_style_from_citeproc_py_styles(self):
        """Test that styles load from citeproc-py-styles when available"""
        # Create a mock module
        mock_module = MagicMock()
        mock_module.get_style_filepath = MagicMock(return_value='/mock/path/to/apa.csl')

        with patch('citeproc.frontend.os.path.exists') as mock_exists:
            # First call checks if 'apa' exists as path (False)
            # Second call checks bundled path (False)
            mock_exists.side_effect = [False, False]

            with patch.dict('sys.modules', {'citeproc_styles': mock_module}):
                # Mock the parent class __init__ to set root properly
                def mock_init(self, path, validate=False):
                    # Create a mock root object 
                    self.root = MagicMock()
                    self.root.get.return_value = 'en-US'
                    self.root.set_locale_list = MagicMock()
                    # Store the path for verification
                    self._test_path = path

                with patch('citeproc.frontend.CitationStylesXML.__init__', mock_init):
                    style = CitationStylesStyle('apa', validate=False)

                    # Verify get_style_filepath was called with correct argument
                    mock_module.get_style_filepath.assert_called_once_with('apa')
                    # Verify parent init was called with the path from citeproc-py-styles
                    self.assertEqual(style._test_path, '/mock/path/to/apa.csl')

    def test_style_not_in_citeproc_py_styles(self):
        """Test error when style not found even in citeproc-py-styles"""
        # Create a mock module that raises KeyError
        mock_module = MagicMock()
        mock_module.get_style_filepath = MagicMock(side_effect=KeyError('Style not found'))

        with patch('citeproc.frontend.os.path.exists') as mock_exists:
            # Style doesn't exist as file or bundled
            mock_exists.return_value = False

            with patch.dict('sys.modules', {'citeproc_styles': mock_module}):
                with self.assertRaises(ValueError) as context:
                    CitationStylesStyle('unknown-style', validate=False)

                error_msg = str(context.exception)
                self.assertIn('unknown-style', error_msg)
                self.assertIn('not found in bundled styles', error_msg)
                self.assertIn('or in citeproc-py-styles package', error_msg)

    def test_style_file_not_found_in_citeproc_py_styles(self):
        """Test error when citeproc-py-styles returns path but file doesn't exist"""
        # Create a mock module that raises FileNotFoundError
        mock_module = MagicMock()
        mock_module.get_style_filepath = MagicMock(side_effect=FileNotFoundError('File not found'))

        with patch('citeproc.frontend.os.path.exists') as mock_exists:
            # Style doesn't exist as file or bundled
            mock_exists.return_value = False

            with patch.dict('sys.modules', {'citeproc_styles': mock_module}):
                with self.assertRaises(ValueError) as context:
                    CitationStylesStyle('missing-style', validate=False)

                error_msg = str(context.exception)
                self.assertIn('missing-style', error_msg)
                self.assertIn('not found in bundled styles', error_msg)
                self.assertIn('or in citeproc-py-styles package', error_msg)

    def test_citeproc_py_styles_load(self):
        """Test loading a real style from citeproc-py-styles if installed"""
        try:
            import citeproc_styles
        except ImportError:
            self.skipTest("citeproc-py-styles (provides citeproc_styles package) not installed")
        from citeproc_styles import get_style_filepath
        # Attempt to load a known style from citeproc-py-styles
        style_name = 'chicago-author-date'
        style_path = get_style_filepath(style_name)
        style = CitationStylesStyle(style_name, validate=False)
        self.assertIsNotNone(style)
        self.assertTrue(hasattr(style, 'root'))
        # And loading with the full path should also work
        style = CitationStylesStyle(style_path, validate=False)
        self.assertIsNotNone(style)
        self.assertTrue(hasattr(style, 'root'))